#!/usr/bin/perl -w
#
# file    : install.pl
# changed : 2005-03-02
# author  : Bas Couwenberg <sebastic@xs4all.nl>
#

use strict;
use File::Basename;
use Getopt::Mixed "nextOption";

# determine environement specific configuration values
# for global configuration variables

my $self    = 'install.pl';
my $version = '0.1';
my $windows = $^O eq 'MSWin32' ? 1 : 0;

my %cfg = ( 
	    self         => $self,
	    version      => $version,
	    install      => 'all',
	    windows      => $windows,
	    ds           => '/',
	    server_dir   => '/var/www/folding/',
	    client_dir   => '/usr/local/fci/',
	    apache_group => 'www-data'
	 );

# change default config based on Operating System
if($cfg{windows}) {
	$cfg{ds}         = '\\';
	$cfg{server_dir} = 'C:\\Program Files\\fci\\www\\';
	$cfg{client_dir} = 'C:\\Program Files\\fci\\client\\';
}

$cfg{source_dir} = dirname($0).$cfg{ds};

# non-standard Perl Modules required for server operation
# this includes the helper scripts
my @server_modules = ( 
			'Date::Calc',
			'Apache::Request',
			'Image::Magick',
			'LWP::UserAgent'
		     );

# non-standard Perl Modules required for client operation
my @client_modules = (
			'LWP::UserAgent',
			'HTTP::Request::Common'
		    );

# get command line parameters
Getopt::Mixed::init("server-dir=s client-dir=s server client verbose help");
while (my ($option, $value) = nextOption()) {
	if($option eq 'help') {
		print_usage();
		Getopt::Mixed::cleanup();
		exit;
	}
	
	$cfg{install}    = $option if($option eq 'server');
	$cfg{install}    = $option if($option eq 'client');
	$cfg{server_dir} = $value  if($option eq 'server-dir');
	$cfg{client_dir} = $value  if($option eq 'client-dir');
	$cfg{verbose}    = 1       if($option eq 'verbose');
}
Getopt::Mixed::cleanup();

# check for requires Perl Modles
# FIXME/CREATEME

# install requisted components
if($cfg{install} eq 'server' || $cfg{install} eq 'all') {
	install_server();
}
if($cfg{install} eq 'client' || $cfg{install} eq 'all') {
	install_client();
}

################################[ Subroutines ]################################

sub print_usage {
	my $verbose = $cfg{verbose} ? 'on' : 'off';

	print "$cfg{self} v$cfg{version}\n\n";
	print "Usage: $cfg{self} [OPTIONS]\n\n";
	print "Options:\n";
	print "--client     Install only client component\n";
	print "--server     Install only server component ";
	print "(includes helper scripts)\n";
	print "--server-dir Directory to install server files\n";
	print "--client-dir Directory to install server files\n";
	print "--verbose    Verbose (debug) output\n";
	print "--help       Print this information\n";
}

sub install_server {
	print "\nStarting installation of Server component.\n";

	# create server directory for server files
	if(!-e $cfg{server_dir}) {
		if(!create_directory($cfg{server_dir}, $cfg{ds})) {
			exit;	
		}
	}
	# copy files to server directory
	
	my $source_dir = $cfg{source_dir}.'www'.$cfg{ds};
	if(-r $source_dir && -w $cfg{server_dir}) {
		print "Copying files from $source_dir to $cfg{server_dir}\n";
		system('cp -r '.$source_dir."* $cfg{server_dir}");

		print "Applying group permissions: $cfg{apache_group}\n";
		system("chgrp -R $cfg{apache_group} $cfg{server_dir}");
	}
	else {
		if(!-r $source_dir) {
			print "Cannot read directory: $source_dir\n";
		}
		elsif(!-w $cfg{server_dir}) {
			print "Cannot write directory: $cfg{server_dir}\n";
		}
		else {
			print "Unspecified Error!\n";
		}
		exit;
	}
	
	# copy helper scripts to a bin direcory
	
	# insert <Directory> directive into httpd.conf
	# add index.pl to <IfModule mod_dir.c> directive
}

sub install_client {
	print "\nStarting installation of Client component.\n";
}

sub create_directory {
	my $dir = $_[0];
	my $ds  = $_[1];

	print "Creating directory: $dir ... ";
	
	my $parent = dirname($dir).$ds;

	# create the directory if the parent dir is writable
	if(-w $parent) {
		mkdir $dir;

		print "Success.\n";
		
		return 1;
	}
	else {	
		print "Error!\n";
		print "Cannot write directory: $parent!\n";
		
		return undef;
	}
}
